@@ -108,6 +108,8 @@ typedef struct _MetaKmsImplDevicePrivate
 
   GHashTable *crtc_frames;
 
+  gboolean realtime_inhibited_pending_mode_set;
+
   MetaDeadlineTimerState deadline_timer_state;
 
   gboolean sync_file_retrieved;
@@ -1470,18 +1472,22 @@ ensure_crtc_frame (MetaKmsImplDevice *impl_device,
   MetaKmsImpl *impl = meta_kms_impl_device_get_impl (impl_device);
   MetaThreadImpl *thread_impl = META_THREAD_IMPL (impl);
   CrtcFrame *crtc_frame;
+  gboolean want_deadline_timer, have_deadline_timer;
 
   crtc_frame = get_crtc_frame (impl_device, latch_crtc);
-  if (crtc_frame)
-    return crtc_frame;
+  if (!crtc_frame)
+    {
+      crtc_frame = g_new0 (CrtcFrame, 1);
+      crtc_frame->impl_device = impl_device;
+      crtc_frame->crtc = latch_crtc;
+      crtc_frame->deadline.timer_fd = -1;
+      crtc_frame->await_flush = TRUE;
+      g_hash_table_insert (priv->crtc_frames, latch_crtc, crtc_frame);
+    }
 
-  crtc_frame = g_new0 (CrtcFrame, 1);
-  crtc_frame->impl_device = impl_device;
-  crtc_frame->crtc = latch_crtc;
-  crtc_frame->deadline.timer_fd = -1;
-  crtc_frame->await_flush = TRUE;
-
-  if (is_using_deadline_timer (impl_device))
+  want_deadline_timer = is_using_deadline_timer (impl_device);
+  have_deadline_timer = crtc_frame->deadline.timer_fd >= 0;
+  if (want_deadline_timer && !have_deadline_timer)
     {
       int timer_fd;
       GSource *source;
@@ -1506,8 +1512,11 @@ ensure_crtc_frame (MetaKmsImplDevice *impl_device,
 
       g_source_unref (source);
     }
-
-  g_hash_table_insert (priv->crtc_frames, latch_crtc, crtc_frame);
+  else if (!want_deadline_timer && have_deadline_timer)
+    {
+      g_clear_fd (&crtc_frame->deadline.timer_fd, NULL);
+      g_clear_pointer (&crtc_frame->deadline.source, g_source_destroy);
+    }
 
   return crtc_frame;
 }
@@ -1688,6 +1697,25 @@ needs_flush:
   meta_kms_device_set_needs_flush (meta_kms_crtc_get_device (crtc), crtc);
 }
 
+static void
+disarm_all_deadline_timers (MetaKmsImplDevice *impl_device)
+{
+  MetaKmsImplDevicePrivate *priv =
+    meta_kms_impl_device_get_instance_private (impl_device);
+  GHashTableIter iter;
+  CrtcFrame *crtc_frame;
+
+  g_hash_table_iter_init (&iter, priv->crtc_frames);
+  while (g_hash_table_iter_next (&iter, NULL, (gpointer *) &crtc_frame))
+    {
+      crtc_frame->deadline.is_deadline_page_flip = FALSE;
+      crtc_frame->await_flush = FALSE;
+      crtc_frame->pending_page_flip = FALSE;
+      g_clear_pointer (&crtc_frame->pending_update, meta_kms_update_free);
+      disarm_crtc_frame_deadline_timer (crtc_frame);
+    }
+}
+
 static MetaKmsFeedback *
 process_mode_set_update (MetaKmsImplDevice *impl_device,
                          MetaKmsUpdate     *update,
@@ -1701,7 +1729,6 @@ process_mode_set_update (MetaKmsImplDevice *impl_device,
   MetaKmsFeedback *feedback;
   CrtcFrame *crtc_frame;
   GList *l;
-  GHashTableIter iter;
 
   for (l = meta_kms_update_get_mode_sets (update); l; l = l->next)
     {
@@ -1720,20 +1747,18 @@ process_mode_set_update (MetaKmsImplDevice *impl_device,
       update = g_steal_pointer (&crtc_frame->pending_update);
     }
 
-  g_hash_table_iter_init (&iter, priv->crtc_frames);
-  while (g_hash_table_iter_next (&iter, NULL, (gpointer *) &crtc_frame))
-    {
-      crtc_frame->deadline.is_deadline_page_flip = FALSE;
-      crtc_frame->await_flush = FALSE;
-      crtc_frame->pending_page_flip = FALSE;
-      g_clear_pointer (&crtc_frame->pending_update, meta_kms_update_free);
-      disarm_crtc_frame_deadline_timer (crtc_frame);
-    }
+  disarm_all_deadline_timers (impl_device);
 
   meta_thread_inhibit_realtime_in_impl (thread);
   feedback = do_process (impl_device, NULL, update, flags);
   meta_thread_uninhibit_realtime_in_impl (thread);
 
+  if (priv->realtime_inhibited_pending_mode_set)
+    {
+      priv->realtime_inhibited_pending_mode_set = FALSE;
+      meta_thread_uninhibit_realtime_in_impl (thread);
+    }
+
   return feedback;
 }
 
@@ -1786,6 +1811,8 @@ meta_kms_impl_device_disable (MetaKmsImplDevice *impl_device)
   if (!priv->device_file)
     return;
 
+  disarm_all_deadline_timers (impl_device);
+
   meta_kms_impl_device_hold_fd (impl_device);
   meta_thread_inhibit_realtime_in_impl (thread);
   klass->disable (impl_device);
@@ -1924,6 +1951,15 @@ meta_kms_impl_device_finalize (GObject *object)
   MetaKmsImplDevicePrivate *priv =
     meta_kms_impl_device_get_instance_private (impl_device);
 
+  if (priv->realtime_inhibited_pending_mode_set)
+    {
+      MetaThreadImpl *thread_impl = META_THREAD_IMPL (priv->impl);
+      MetaThread *thread = meta_thread_impl_get_thread (thread_impl);
+
+      priv->realtime_inhibited_pending_mode_set = FALSE;
+      meta_thread_uninhibit_realtime_in_impl (thread);
+    }
+
   meta_kms_impl_remove_impl_device (priv->impl, impl_device);
 
   g_list_free_full (priv->planes, g_object_unref);
@@ -1973,6 +2009,16 @@ meta_kms_impl_device_init_mode_setting (MetaKmsImplDevice  *impl_device,
 
   update_connectors (impl_device, drm_resources, 0);
 
+  if (!priv->crtcs)
+    {
+      MetaThreadImpl *thread_impl = META_THREAD_IMPL (priv->impl);
+      MetaThread *thread = meta_thread_impl_get_thread (thread_impl);
+
+      g_warn_if_fail (priv->realtime_inhibited_pending_mode_set);
+      meta_thread_uninhibit_realtime_in_impl (thread);
+      priv->realtime_inhibited_pending_mode_set = FALSE;
+    }
+
   drmModeFreeResources (drm_resources);
 
   return TRUE;
@@ -2050,6 +2096,8 @@ meta_kms_impl_device_initable_init (GInitable     *initable,
   MetaKmsImplDevice *impl_device = META_KMS_IMPL_DEVICE (initable);
   MetaKmsImplDevicePrivate *priv =
     meta_kms_impl_device_get_instance_private (impl_device);
+  MetaThreadImpl *thread_impl = META_THREAD_IMPL (priv->impl);
+  MetaThread *thread = meta_thread_impl_get_thread (thread_impl);
   int fd;
 
   if (!ensure_device_file (impl_device, error))
@@ -2075,6 +2123,9 @@ meta_kms_impl_device_initable_init (GInitable     *initable,
 
   priv->sync_file = -1;
 
+  meta_thread_inhibit_realtime_in_impl (thread);
+  priv->realtime_inhibited_pending_mode_set = TRUE;
+
   return TRUE;
 }
 
