@@ -226,7 +226,7 @@ meta_wayland_touch_update (MetaWaylandTouch   *touch,
   sequence = clutter_event_get_event_sequence (event);
   event_type = clutter_event_type (event);
 
-  if (event_type == CLUTTER_TOUCH_BEGIN)
+  if (event_type == CLUTTER_ENTER)
     {
       MetaWaylandSurface *surface = NULL;
       MetaBackend *backend;
@@ -256,7 +256,8 @@ meta_wayland_touch_update (MetaWaylandTouch   *touch,
   if (!touch_info)
     return;
 
-  if (event_type != CLUTTER_TOUCH_BEGIN &&
+  if ((event_type == CLUTTER_TOUCH_UPDATE ||
+       event_type == CLUTTER_TOUCH_END) &&
       !touch_info->begin_delivered)
     {
       g_hash_table_remove (touch->touches, sequence);
